@@ -1,6 +1,6 @@
-/* 
+/*
  *  Copyright (C) 1989,2004 by the Massachusetts Institute of Technology
- * 
+ *
  * WITHIN THAT CONSTRAINT, permission to use, copy, modify, and
  * distribute this software and its documentation for any purpose and
  * without fee is hereby granted, provided that the above copyright
@@ -67,18 +67,19 @@
 #include <krb5.h>
 #include <afs/ptserver.h>
 #include <afs/ptuser.h>
+#include <afs/pterror.h>
 
 #ifdef WIN32
 #include <windows.h>
 
-#include <cm_config.h>
-#include <auth.h>
-#include <cellconfig.h>
-#include <pioctl_nt.h>
-#include <smb_iocons.h>
+#include <afs\cm_config.h>
+#include <afs\auth.h>
+#include <afs\cellconfig.h>
+#include <afs\pioctl_nt.h>
+#include <afs\smb_iocons.h>
+#include <WINNT\afsreg.h>
 
 #define stat _stat
-#define lstat stat
 #define __S_ISTYPE(mode, mask) (((mode) & _S_IFMT) == (mask))
 #define S_ISDIR(mode)          __S_ISTYPE((mode), _S_IFDIR)
 
@@ -95,7 +96,7 @@
 #define DRIVECOLON ':'		/* Drive letter separator */
 #define BDIR '\\'		/* Other character that divides directories */
 
-static int 
+static int
 readlink(char *path, char *buf, int buffers)
 {
 	return -1;
@@ -103,7 +104,7 @@ readlink(char *path, char *buf, int buffers)
 
 char * getcwd(char*, size_t);
 
-static long 
+static long
 get_cellconfig_callback(void *cellconfig, struct sockaddr_in *addrp, char *namep)
 {
 	struct afsconf_cell *cc = (struct afsconf_cell *) cellconfig;
@@ -254,21 +255,21 @@ long GetLocalCell(struct afsconf_dir **pconfigdir, char *local_cell)
     return afsconf_GetLocalCell(*pconfigdir, local_cell, MAXCELLCHARS);
 }
 
-long GetCellInfo(struct afsconf_dir **pconfigdir, char* cell, 
+long GetCellInfo(struct afsconf_dir **pconfigdir, char* cell,
 struct afsconf_cell **pcellconfig)
 {
     return afsconf_GetCellInfo(*pconfigdir, cell, NULL, *pcellconfig);
 }
 
 void CloseConf(struct afsconf_dir **pconfigdir)
-{       
+{
     (void) afsconf_Close(*pconfigdir);
 }
 
 #define ALLOW_REGISTER 1
 void ViceIDToUsername(char *username, char *realm_of_user, char *realm_of_cell,
                       char * cell_to_use, CREDENTIALS *c,
-                      int *status, 
+                      int *status,
                       struct ktc_principal *aclient, struct ktc_principal *aserver, struct ktc_token *atoken)
 {
     static char lastcell[MAXCELLCHARS+1] = { 0 };
@@ -301,10 +302,10 @@ void ViceIDToUsername(char *username, char *realm_of_user, char *realm_of_cell,
     if (dflag)
     {
         if (*status)
-            printf("Error %d\n", *status);
+            printf("pr_SNameToId Error %s\n",  afs_error_message(*status));
         else
             printf("Id %d\n", viceId);
-    }       
+    }
 
     /*
      * This code is taken from cklog -- it lets people
@@ -357,25 +358,23 @@ void ViceIDToUsername(char *username, char *realm_of_user, char *realm_of_cell,
              */
 
             if ((*status = pr_Initialize(1L, confname, aserver->cell))) {
-                printf("Error %d\n", *status);
+                printf("pr_Initialize Error %s\n",  afs_error_message(*status));
                 return;
             }
 
             /* copy the name because pr_CreateUser lowercases the realm */
             strncpy(username_copy, username, BUFSIZ);
 
-            *status = pr_CreateUser(username, &viceId);
-
-            /* and restore the name to the original state */
-            strncpy(username, username_copy, BUFSIZ);
+            viceId = 0;
+            *status = pr_CreateUser(username_copy, &viceId);
 
             if (*status) {
                 printf("%s: unable to create remote PTS "
-                        "user %s in cell %s (status: %d).\n", progname,
-                        username, cell_to_use, *status);
+                        "user %s in cell %s (status: %s).\n", progname,
+                        username_copy, cell_to_use, afs_error_message(*status));
             } else {
                 printf("created cross-cell entry for %s (Id %d) at %s\n",
-                        username, viceId, cell_to_use);
+                        username_copy, viceId, cell_to_use);
 #ifdef AFS_ID_TO_NAME
                 snprintf (username, BUFSIZ, "%s (AFS ID %d)", username_copy, (int) viceId);
 #endif /* AFS_ID_TO_NAME */
@@ -460,13 +459,13 @@ static int get_cred(char *name, char *inst, char *realm, CREDENTIALS *c)
 #endif
         if (status == KSUCCESS)
             status = krb_get_cred(name, inst, realm, c);
-    }       
+    }
 
     return (status);
 }
 #endif
 
-static int get_v5cred(krb5_context context, 
+static int get_v5cred(krb5_context context,
                       char *name, char *inst, char *realm, CREDENTIALS *c,
                       krb5_creds **creds)
 {
@@ -548,8 +547,8 @@ static char *afs_realm_of_cell(struct afsconf_cell *cellconfig)
 }
 #endif
 
-/* As of MIT Kerberos 1.6, krb5_get_host_realm() will return the NUL-string 
- * if there is no domain_realm mapping for the hostname's domain.  This is 
+/* As of MIT Kerberos 1.6, krb5_get_host_realm() will return the NUL-string
+ * if there is no domain_realm mapping for the hostname's domain.  This is
  * used as a trigger indicating that referrals should be used within the
  * krb5_get_credentials() call.  However, if the KDC does not support referrals
  * that will result in a KRB5_ERR_HOST_REALM_UNKNOWN error and we will have
@@ -586,7 +585,7 @@ static char *afs_realm_of_cell5(krb5_context context, struct afsconf_cell *cellc
 	    krb5_free_host_realm( context, krbrlms );
     }
     return krbrlm;
-}	
+}
 
 static char *copy_cellinfo(cellinfo_t *cellinfo)
 {
@@ -674,6 +673,65 @@ copy_realm_of_ticket(krb5_context context, char * dest, size_t destlen, krb5_cre
     }
 }
 
+typedef BOOL (WINAPI *LPFN_ISWOW64PROCESS) (HANDLE, PBOOL);
+static
+int is_wow64()
+{
+    static int init = TRUE;
+    static int bIsWow64 = FALSE;
+
+    if (init) {
+        HMODULE hModule;
+        LPFN_ISWOW64PROCESS fnIsWow64Process = NULL;
+
+        hModule = GetModuleHandle(TEXT("kernel32"));
+        if (hModule) {
+            fnIsWow64Process = (LPFN_ISWOW64PROCESS)GetProcAddress(hModule, "IsWow64Process");
+
+            if (NULL != fnIsWow64Process)
+            {
+                if (!fnIsWow64Process(GetCurrentProcess(),&bIsWow64))
+                {
+                    // on error, assume FALSE.
+                    // in other words, do nothing.
+                }
+            }
+            FreeLibrary(hModule);
+        }
+        init = FALSE;
+    }
+    return bIsWow64;
+}
+
+static int
+accept_dotted_usernames(void)
+{
+    HKEY parmKey;
+    DWORD code, len;
+    DWORD value = 1;
+
+    code = RegOpenKeyEx(HKEY_CURRENT_USER, AFSREG_USER_OPENAFS_SUBKEY,
+                         0, (is_wow64()?KEY_WOW64_64KEY:0)|KEY_QUERY_VALUE, &parmKey);
+    if (code == ERROR_SUCCESS) {
+        len = sizeof(value);
+        code = RegQueryValueEx(parmKey, "AcceptDottedPrincipalNames", NULL, NULL,
+                                (BYTE *) &value, &len);
+        RegCloseKey(parmKey);
+    }
+    if (code != ERROR_SUCCESS) {
+        code = RegOpenKeyEx(HKEY_LOCAL_MACHINE, AFSREG_CLT_OPENAFS_SUBKEY,
+                             0, (is_wow64()?KEY_WOW64_64KEY:0)|KEY_QUERY_VALUE, &parmKey);
+        if (code == ERROR_SUCCESS) {
+            len = sizeof(value);
+            code = RegQueryValueEx(parmKey, "AcceptDottedPrincipalNames", NULL, NULL,
+                                    (BYTE *) &value, &len);
+            RegCloseKey (parmKey);
+        }
+    }
+    return value;
+}
+
+
 /*
 * Log to a cell.  If the cell has already been logged to, return without
 * doing anything.  Otherwise, log to it and mark that it has been logged
@@ -754,14 +812,14 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
      * afs style authenticator.
      */
 
-    if (usev5) 
+    if (usev5)
     { /* using krb5 */
         int retry = 1;
 	int realm_fallback = 0;
 
         if ((status = get_v5_user_realm(context, realm_of_user)) != KSUCCESS) {
             char * msg;
-            
+
             if (pkrb5_get_error_message)
                 msg = pkrb5_get_error_message(context, status);
             else
@@ -774,7 +832,7 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
             goto done;
         }
 
-        if ( strchr(name,'.') != NULL ) {
+        if ( strchr(name,'.') != NULL && !accept_dotted_usernames()) {
             fprintf(stderr, "%s: Can't support principal names including a dot.\n",
                     progname);
             status = AKLOG_MISC;
@@ -785,9 +843,9 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
 	if (realm && realm[0]) {
             if (dflag)
                 printf("Getting v5 tickets: %s/%s@%s\n", name, instance, realm);
-            status = get_v5cred(context, name, instance, realm, 
+            status = get_v5cred(context, name, instance, realm,
 #ifdef HAVE_KRB4
-                            use524 ? &c : NULL, 
+                            use524 ? &c : NULL,
 #else
                             NULL,
 #endif
@@ -802,15 +860,15 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
                 status = -1;
                 if (dflag)
                     printf("Getting v5 tickets: %s/%s@%s\n", name, instance, realm_of_user);
-                status = get_v5cred(context, name, instance, realm_of_user, 
+                status = get_v5cred(context, name, instance, realm_of_user,
 #ifdef HAVE_KRB4
-                                     use524 ? &c : NULL, 
+                                     use524 ? &c : NULL,
 #else
                                      NULL,
 #endif
                                      &v5cred);
                 if (status == 0) {
-                    /* we have determined that the client realm 
+                    /* we have determined that the client realm
                      * is a valid cell realm
                      */
                     strcpy(realm_of_cell, realm_of_user);
@@ -820,14 +878,14 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
             if (status != 0 && (!retry || retry && strcmp(realm_of_user,realm_of_cell))) {
                 if (dflag)
                     printf("Getting v5 tickets: %s/%s@%s\n", name, instance, realm_of_cell);
-                status = get_v5cred(context, name, instance, realm_of_cell, 
+                status = get_v5cred(context, name, instance, realm_of_cell,
 #ifdef HAVE_KRB4
-                                     use524 ? &c : NULL, 
+                                     use524 ? &c : NULL,
 #else
                                      NULL,
 #endif
                                      &v5cred);
-                if (!status && !strlen(realm_of_cell)) 
+                if (!status && !strlen(realm_of_cell))
                     copy_realm_of_ticket(context, realm_of_cell, sizeof(realm_of_cell), v5cred);
             }
         }
@@ -842,24 +900,24 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
 	    }
             if (dflag)
                 printf("Getting v5 tickets: %s@%s\n", name, realm_of_cell);
-            status = get_v5cred(context, name, "", realm_of_cell, 
+            status = get_v5cred(context, name, "", realm_of_cell,
 #ifdef HAVE_KRB4
-                                use524 ? &c : NULL, 
+                                use524 ? &c : NULL,
 #else
                                 NULL,
 #endif
                                 &v5cred);
-            if (!status && !strlen(realm_of_cell)) 
+            if (!status && !strlen(realm_of_cell))
                 copy_realm_of_ticket(context, realm_of_cell, sizeof(realm_of_cell), v5cred);
 	}
-     
+
         if ( status == KRB5KRB_AP_ERR_MSG_TYPE && retry ) {
             retry = 0;
 	    realm_fallback = 0;
             goto try_v5;
-        }       
-    }       
-    else 
+        }
+    }
+    else
     {
 #ifdef HAVE_KRB4
 	if (realm && realm[0])
@@ -887,7 +945,7 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
         status = AKLOG_MISC;
         goto done;
 #endif
-    } 
+    }
 
     if (status != KSUCCESS)
     {
@@ -924,7 +982,7 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
          */
         char * p;
         int len;
-        
+
         len = min(v5cred->client->data[0].length,MAXKTCNAMELEN - 1);
         strncpy(username, v5cred->client->data[0].data, len);
         username[len] = '\0';
@@ -973,7 +1031,7 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
          atoken.ticketLen == btoken.ticketLen &&
          !memcmp(&atoken.sessionKey, &btoken.sessionKey, sizeof(atoken.sessionKey)) &&
          !memcmp(atoken.ticket, btoken.ticket, atoken.ticketLen))
-    {       
+    {
         if (dflag)
             printf("Identical tokens already exist; skipping.\n");
         status = AKLOG_SUCCESS;
@@ -984,8 +1042,8 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
     {
         if (dflag)
             printf("Not resolving name %s to id (-noprdb set)\n", username);
-    }       
-    else    
+    }
+    else
     {
         if (!usev5) {
 #ifdef HAVE_KRB4
@@ -1006,9 +1064,9 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
         strcat(username, "@");
         strcat(username, realm_of_user);
 
-        ViceIDToUsername(username, realm_of_user, realm_of_cell, cell_to_use, 
+        ViceIDToUsername(username, realm_of_user, realm_of_cell, cell_to_use,
 #ifdef HAVE_KRB4
-                          &c, 
+                          &c,
 #else
                           NULL,
 #endif
@@ -1024,12 +1082,12 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
      */
     strncpy(aclient.name, username, MAXKTCNAMELEN - 1);
     strcpy(aclient.instance, "");
-    
+
     if (usev5 && !use524) {
         int len = min(v5cred->client->realm.length,MAXKTCNAMELEN - 1);
         strncpy(aclient.cell, v5cred->client->realm.data, len);
         aclient.cell[len] = '\0';
-    } 
+    }
 #ifdef HAVE_KRB4
     else
 	strncpy(aclient.cell, c.realm, MAXKTCREALMLEN - 1);
@@ -1058,7 +1116,7 @@ static int auth_to_cell(krb5_context context, char *cell, char *realm)
 
   done:
 #if 0
-    /* 
+    /*
      * intentionally leak the linkedCell field because it was allocated
      * using a different C RTL version.
      */
@@ -1119,7 +1177,7 @@ static int get_afs_mountpoint(char *file, char *mountpoint, int size)
     else {
         return(FALSE);
     }
-}       
+}
 
 /*
 * This routine each time it is called returns the next directory
@@ -1212,7 +1270,7 @@ static char *next_path(char *origpath)
         }
         else
             last_comp = elast_comp;
-    }       
+    }
     while(link);
 
     return(pathtocheck);
@@ -1251,9 +1309,9 @@ static int auth_to_path(krb5_context context, char *path)
             /* in WIN32, if getcwd returns a root dir (eg: c:\), the returned string
             * will already have a trailing slash ('\'). Otherwise, the string will
             * end in the last directory name */
-#ifdef WIN32    
+#ifdef WIN32
             if(pathtocheck[strlen(pathtocheck) - 1] != BDIR)
-#endif  
+#endif
                 strcat(pathtocheck, DIRSTRING);
             strcat(pathtocheck, path);
         }
@@ -1367,7 +1425,7 @@ void
 validate_krb5_availability(void)
 {
     HINSTANCE h = LoadLibrary(KRB5LIB);
-    if (h) 
+    if (h)
         FreeLibrary(h);
     else {
         fprintf(stderr, "Kerberos for Windows library %s is not available.\n", KRB5LIB);
@@ -1380,7 +1438,7 @@ validate_krb4_availability(void)
 {
 #ifdef HAVE_KRB4
     HINSTANCE h = LoadLibrary("krbv4w32.dll");
-    if (h) 
+    if (h)
         FreeLibrary(h);
     else {
         fprintf(stderr, "Kerberos for Windows library krbv4w32.dll is not available.\n");
@@ -1452,7 +1510,7 @@ int main(int argc, char *argv[])
             force++;
         else if (((strcmp(argv[i], "-cell") == 0) ||
                    (strcmp(argv[i], "-c") == 0)) && !pmode)
-        {       
+        {
             if (++i < argc)
             {
                 cmode++;
@@ -1463,7 +1521,7 @@ int main(int argc, char *argv[])
         }
         else if (((strcmp(argv[i], "-path") == 0) ||
                    (strcmp(argv[i], "-p") == 0)) && !cmode)
-        {       
+        {
             if (++i < argc)
             {
                 pmode++;
@@ -1552,12 +1610,15 @@ int main(int argc, char *argv[])
         }
     }
 
+    if (!noprdb)
+        initialize_PT_error_table();
+
     if (usev5) {
         validate_krb5_availability();
         if (krb5_init_context(&context))
             return(AKLOG_KERBEROS);
         load_krb5_error_message_funcs();
-    } else 
+    } else
         validate_krb4_availability();
     afs_set_com_err_hook(redirect_errors);
 
@@ -1570,15 +1631,15 @@ int main(int argc, char *argv[])
         for (cur_node = cells.first; cur_node; cur_node = cur_node->next)
         {
             memcpy(&cellinfo, cur_node->data, sizeof(cellinfo));
-            if (status = auth_to_cell(context, 
+            if (status = auth_to_cell(context,
                                        cellinfo.cell, cellinfo.realm))
                 somethingswrong++;
-        }       
+        }
 
         /* Then, log to all paths in the paths list */
         for (cur_node = paths.first; cur_node; cur_node = cur_node->next)
         {
-            if (status = auth_to_path(context, 
+            if (status = auth_to_path(context,
                                        cur_node->data))
                 somethingswrong++;
         }
@@ -1590,7 +1651,7 @@ int main(int argc, char *argv[])
         */
         if (somethingswrong && ((cells.nelements + paths.nelements) > 1))
             status = AKLOG_SOMETHINGSWRONG;
-    }       
+    }
 
     akexit(status);
-}       
+}
