@@ -1,18 +1,21 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
+#include "stdafx.h"
+#include <winsock2.h>
+#include <ws2tcpip.h>
+
 extern "C" {
 #include <afs/param.h>
 #include <afs/stds.h>
 }
 
-#include "stdafx.h"
 #include <errno.h>
 #include <time.h>
 
@@ -29,13 +32,19 @@ extern "C" {
 #include <rx/rx_globals.h>
 #include "fs.h"
 #include "fs_utils.h"
-#include <afsint.h>
+#include <afs/afsint.h>
+#include <afs/afs_consts.h>
 #include <afs/cellconfig.h>
 #include <afs/vldbint.h>
 #include <afs/volser.h>
 #include <afs/auth.h>
 #include <WINNT\afsreg.h>
 #include <cm.h>
+#include <cm_nls.h>
+#include <osi.h>
+#include <cm_user.h>
+#include <cm_scache.h>
+#include <cm_ioctl.h>
 }
 
 #define STRSAFE_NO_DEPRECATE
@@ -44,10 +53,7 @@ extern "C" {
 #define PCCHAR(str)		((char *)(const char *)(str))
 #define VL_NOENT                (363524L)
 
-#define	MAXHOSTS 13
-#define	OMAXHOSTS 8
 #define MAXNAME 100
-#define	MAXSIZE	2048
 #define MAXINSIZE 1300    /* pioctl complains if data is larger than this */
 #define VMSGSIZE 128      /* size of msg buf in volume hdr */
 
@@ -55,9 +61,6 @@ extern "C" {
 #define MAXHOSTCHARS		64
 #define MAXHOSTSPERCELL		8
 
-static char space[MAXSIZE];
-static char tspace[1024];
-
 static struct ubik_client *uclient;
 static int rxInitDone = 0;
 static char pn[] = "fs";
@@ -150,8 +153,8 @@ VLDBInit(int noAuthFlag, struct afsconf_cell *info)
 {
     afs_int32 code;
 
-    code = ugen_ClientInit(noAuthFlag, (char *)AFSDIR_CLIENT_ETC_DIRPATH, 
-			   info->name, 0, &uclient, 
+    code = ugen_ClientInit(noAuthFlag, (char *)AFSDIR_CLIENT_ETC_DIRPATH,
+			   info->name, 0, &uclient,
                            NULL, pn, rxkad_clear,
                            VLDB_MAXSERVERS, AFSCONF_VLDBSERVICE, 50,
                            0, 0, USER_SERVICE_ID);
@@ -168,7 +171,7 @@ OpenFile(char *file, char *rwp)
     FILE *fp;
 
     code = GetWindowsDirectoryA(wdir, sizeof(wdir));
-    if (code == 0 || code > sizeof(wdir)) 
+    if (code == 0 || code > sizeof(wdir))
         return FALSE;
 
     /* add trailing backslash, if required */
@@ -181,7 +184,7 @@ OpenFile(char *file, char *rwp)
     fp = fopen(wdir, rwp);
 
     return fp;
-}       
+}
 
 CString StripPath(CString& strPath)
 {
@@ -204,7 +207,7 @@ CStringArray& StripPath(CStringArray& files)
 
 void Flush(const CStringArray& files)
 {
-    register LONG code;
+    LONG code;
     struct ViceIoctl blob;
     int error = 0;
 
@@ -218,18 +221,18 @@ void Flush(const CStringArray& files)
             error = 1;
             if (errno == EMFILE)
                 ShowMessageBox(IDS_FLUSH_FAILED, MB_ICONERROR, IDS_FLUSH_FAILED, files[i]);
-            else 
+            else
                 ShowMessageBox(IDS_FLUSH_ERROR, MB_ICONERROR, IDS_FLUSH_ERROR, files[i], strerror(errno));
         }
-    }   
+    }
 
     if (!error)
         ShowMessageBox(IDS_FLUSH_OK, MB_ICONINFORMATION, IDS_FLUSH_OK);
-}       
+}
 
 void FlushVolume(const CStringArray& files)
 {
-    register LONG code;
+    LONG code;
     struct ViceIoctl blob;
     int error = 0;
 
@@ -243,15 +246,15 @@ void FlushVolume(const CStringArray& files)
             error = 1;
             ShowMessageBox(IDS_FLUSH_VOLUME_ERROR, MB_ICONERROR, IDS_FLUSH_VOLUME_ERROR, files[i], strerror(errno));
         }
-    }   
+    }
 
     if (!code)
         ShowMessageBox(IDS_FLUSH_VOLUME_OK, MB_ICONINFORMATION, IDS_FLUSH_VOLUME_OK);
-}       
+}
 
 void WhichCell(CStringArray& files)
 {
-    register LONG code;
+    LONG code;
     struct ViceIoctl blob;
     int error;
     CString str;
@@ -264,8 +267,10 @@ void WhichCell(CStringArray& files)
     HOURGLASS hourglass;
 
     for (int i = 0; i < files.GetSize(); i++) {
+        char space[AFS_PIOCTL_MAXSIZE];
+
         blob.in_size = 0;
-        blob.out_size = MAXSIZE;
+        blob.out_size = AFS_PIOCTL_MAXSIZE;
         blob.out = space;
 
         code = pioctl_T(files[i], VIOC_FILE_CELL_NAME, &blob, 1);
@@ -276,9 +281,10 @@ void WhichCell(CStringArray& files)
             } else
                 results.Add(GetAfsError(errno));
         } else {
+            space[AFS_PIOCTL_MAXSIZE - 1] = '\0';
             results.Add(Utf8ToCString(space));
         }
-    }       
+    }
 
     LoadString (str, IDS_SHOW_CELL);
     LoadString (str2, IDS_SHOW_CELL_COLUMN);
@@ -289,14 +295,15 @@ void WhichCell(CStringArray& files)
 
 void WSCellCmd()
 {
-    register LONG code;
+    char space[AFS_PIOCTL_MAXSIZE];
+    LONG code;
     struct ViceIoctl blob;
-    
+
     HOURGLASS hourglass;
 
     blob.in_size = 0;
     blob.in = (char *) 0;
-    blob.out_size = MAXSIZE;
+    blob.out_size = AFS_PIOCTL_MAXSIZE;
     blob.out = space;
 
     code = pioctl((char *) 0, VIOC_GET_WS_CELL, &blob, 1);
@@ -310,9 +317,9 @@ void WSCellCmd()
 
 BOOL CheckVolumes()
 {
-    register LONG code;
+    LONG code;
     struct ViceIoctl blob;
-    
+
     blob.in_size = 0;
     blob.out_size = 0;
     code = pioctl(0, VIOCCKBACK, &blob, 1);
@@ -328,9 +335,9 @@ BOOL CheckVolumes()
 
 void SetCacheSizeCmd(LONG nNewCacheSize)
 {
-    register LONG code;
+    LONG code;
     struct ViceIoctl blob;
-    
+
     HOURGLASS hourglass;
 
     blob.in = (char *) &nNewCacheSize;
@@ -346,7 +353,7 @@ void SetCacheSizeCmd(LONG nNewCacheSize)
 
 void WhereIs(CStringArray& files)
 {
-    register LONG code;
+    LONG code;
     struct ViceIoctl blob;
     CStringArray servers;
     CStringArray resultFiles;
@@ -356,7 +363,9 @@ void WhereIs(CStringArray& files)
     HOURGLASS hourglass;
 
     for (int i = 0; i < files.GetSize(); i++) {
-        blob.out_size = MAXSIZE;
+        char space[AFS_PIOCTL_MAXSIZE];
+
+        blob.out_size = AFS_PIOCTL_MAXSIZE;
         blob.in_size = 0;
         blob.out = space;
         memset(space, 0, sizeof(space));
@@ -372,7 +381,7 @@ void WhereIs(CStringArray& files)
         BOOL bFirst = TRUE;
         str = "";
 
-        for (int j = 0; j < MAXHOSTS; j++) {
+        for (int j = 0; j < AFS_MAXHOSTS; j++) {
             if (hosts[j] == 0)
                 break;
             char *hostName = hostutil_GetNameByINet(hosts[j]);
@@ -390,7 +399,7 @@ void WhereIs(CStringArray& files)
     CResultsDlg dlg(SHOW_FILE_SERVERS_HELP_ID);
     dlg.SetContents(str, str2, resultFiles, servers);
     dlg.DoModal();
-}       
+}
 
 static int
 CMtoUNIXerror(int cm_code)
@@ -443,12 +452,12 @@ CString GetAfsError(int code, const TCHAR *filename)
     if (code == EINVAL) {
         if (filename)
             strMsg.Format(_T("Invalid argument; it is possible that the file is not in AFS"));
-        else 
+        else
             strMsg.Format(_T("Invalid argument"));
     } else if (code == ENOENT) {
-        if (filename) 
+        if (filename)
             strMsg.Format(_T("The file does not exist"));
-        else 
+        else
             strMsg.Format(_T("No such file returned"));
     } else if (code == EROFS)  {
         strMsg.Format(_T("You can not change a backup or readonly volume"));
@@ -493,9 +502,9 @@ struct Acl {
     struct AclEntry *minuslist;
 };
 
-int foldcmp (register char *a, register char *b)
+int foldcmp (char *a, char *b)
 {
-    register char t, u;
+    char t, u;
     while (1) {
         t = *a++;
         u = *b++;
@@ -508,7 +517,7 @@ int foldcmp (register char *a, register char *b)
 
 extern "C" void ZapList(struct AclEntry *alist)
 {
-    register struct AclEntry *tp, *np;
+    struct AclEntry *tp, *np;
 
     for (tp = alist; tp; tp = np) {
         np = tp->next;
@@ -528,7 +537,7 @@ extern "C" int PruneList (struct AclEntry **ae, int dfs)
     struct AclEntry **lp = ae;
     struct AclEntry *te, *ne;
     LONG ctr = 0;
-    
+
     for (te = *ae; te; te = ne) {
         if ((!dfs && te->rights == 0) || te->rights == -1) {
             *lp = te->next;
@@ -541,35 +550,35 @@ extern "C" int PruneList (struct AclEntry **ae, int dfs)
             lp = &te->next;
         }
     }
-    
+
     return ctr;
 }
 
-char *SkipLine (register char *astr)
+char *SkipLine (char *astr)
 {
-    while (*astr != '\n') 
+    while (*astr != '\n')
         astr++;
-    
+
     astr++;
-    
+
     return astr;
 }
 
 /* tell if a name is 23 or -45 (digits or minus digits), which are bad names we must prune */
-static int BadName(register char *aname)
+static int BadName(char *aname)
 {
-    register int tc;
+    int tc;
 
     /* all must be '-' or digit to be bad */
     while (tc = *aname++) {
-        if ((tc != '-') && (tc < '0' || tc > '9')) 
+        if ((tc != '-') && (tc < '0' || tc > '9'))
             return 0;
     }
 
     return 1;
 }
 
-CString GetRightsString(register LONG arights, int dfs)
+CString GetRightsString(LONG arights, int dfs)
 {
     CString str;
 
@@ -592,42 +601,42 @@ CString GetRightsString(register LONG arights, int dfs)
 		if (arights & DFS_DELETE) str += _T("d"); else printf(_T("-"));
 		if (arights & (DFS_USRALL)) str += _T("+");
 */
-    }	
+    }
 
     return str;
 }
 
 char *AclToString(struct Acl *acl)
 {
-    static char mydata[MAXSIZE];
-    char tstring[MAXSIZE];
+    static char mydata[AFS_PIOCTL_MAXSIZE];
+    char tstring[AFS_PIOCTL_MAXSIZE];
     char dfsstring[30];
     struct AclEntry *tp;
-    
+
     if (acl->dfs)
         sprintf(dfsstring, " dfs:%d %s", acl->dfs, acl->cell);
     else
         dfsstring[0] = '\0';
     sprintf(mydata, "%d%s\n%d\n", acl->nplus, dfsstring, acl->nminus);
-    
+
     for(tp = acl->pluslist; tp; tp = tp->next) {
         sprintf(tstring, "%s %d\n", tp->name, tp->rights);
         strcat(mydata, tstring);
     }
-    
+
     for(tp = acl->minuslist; tp; tp = tp->next) {
         sprintf(tstring, "%s %d\n", tp->name, tp->rights);
         strcat(mydata, tstring);
     }
-    
+
     return mydata;
 }
 
 struct Acl *EmptyAcl(const CString& strCellName)
 {
-    register struct Acl *tp;
+    struct Acl *tp;
     CStringUtf8 ustrCell(strCellName);
-    
+
     tp = (struct Acl *)malloc(sizeof (struct Acl));
     tp->nplus = tp->nminus = 0;
     tp->pluslist = tp->minuslist = 0;
@@ -639,7 +648,7 @@ struct Acl *EmptyAcl(const CString& strCellName)
 
 struct Acl *EmptyAcl(char *astr)
 {
-    register struct Acl *tp;
+    struct Acl *tp;
     int junk;
 
     tp = (struct Acl *)malloc(sizeof (struct Acl));
@@ -684,19 +693,19 @@ ParseAcl (char *astr)
     last = 0;
     first = 0;
     for(i=0;i<nplus;i++) {
-        ret = sscanf(astr, "%100s %d", tname, &trights); 
+        ret = sscanf(astr, "%100s %d", tname, &trights);
         if (ret <= 0)
             goto nplus_err;
         astr = SkipLine(astr);
         tl = (struct AclEntry *) malloc(sizeof (struct AclEntry));
         if (tl == NULL)
             goto nplus_err;
-        if (!first) 
+        if (!first)
             first = tl;
         strcpy(tl->name, tname);
         tl->rights = trights;
         tl->next = 0;
-        if (last) 
+        if (last)
             last->next = tl;
         last = tl;
     }
@@ -712,12 +721,12 @@ ParseAcl (char *astr)
         tl = (struct AclEntry *) malloc(sizeof (struct AclEntry));
         if (tl == NULL)
             goto nminus_err;
-        if (!first) 
+        if (!first)
             first = tl;
         strcpy(tl->name, tname);
         tl->rights = trights;
         tl->next = 0;
-        if (last) 
+        if (last)
             last->next = tl;
         last = tl;
     }
@@ -729,14 +738,14 @@ ParseAcl (char *astr)
     for (;first; first = next) {
         next = first->next;
         free(first);
-    }   
+    }
     first = ta->pluslist;
 
   nplus_err:
     for (;first; first = next) {
         next = first->next;
         free(first);
-    }   
+    }
     free(ta);
     return NULL;
 }
@@ -745,7 +754,7 @@ ParseAcl (char *astr)
    any changes to the list, and 0 otherwise */
 extern "C" int CleanAcl(struct Acl *aa)
 {
-    register struct AclEntry *te, **le, *ne;
+    struct AclEntry *te, **le, *ne;
     int changes;
 
     HOURGLASS hourglass;
@@ -771,7 +780,7 @@ extern "C" int CleanAcl(struct Acl *aa)
     }
 
     le = &aa->minuslist;
-    
+
     for(te = aa->minuslist; te; te = ne) {
         ne = te->next;
         if (BadName(te->name)) {
@@ -783,15 +792,15 @@ extern "C" int CleanAcl(struct Acl *aa)
         }
         else
             le = &te->next;
-    }   
+    }
 
     return changes;
 }
 
 void CleanACL(CStringArray& names)
 {
-    register LONG code;
-    register struct Acl *ta;
+    LONG code;
+    struct Acl *ta;
     struct ViceIoctl blob;
     int changes;
 
@@ -800,7 +809,9 @@ void CleanACL(CStringArray& names)
     HOURGLASS hourglass;
 
     for (int i = 0; i < names.GetSize(); i++) {
-        blob.out_size = MAXSIZE;
+        char space[AFS_PIOCTL_MAXSIZE];
+
+        blob.out_size = AFS_PIOCTL_MAXSIZE;
         blob.in_size = 0;
         blob.out = space;
 
@@ -828,7 +839,7 @@ void CleanACL(CStringArray& names)
         blob.in = AclToString(ta);
         blob.in_size = strlen((char *)blob.in) + 1;
         blob.out_size = 0;
-		
+
         code = pioctl_T(names[i], VIOCSETAL, &blob, 1);
         if (code) {
             if (errno == EINVAL) {
@@ -841,23 +852,23 @@ void CleanACL(CStringArray& names)
             }
         }
     }
-}       
+}
 
 // Derived from fs.c's ListAclCmd
 BOOL GetRights(const CString& strDir, CStringArray& strNormal, CStringArray& strNegative)
 {
-    register LONG code;
-    register struct Acl *ta;
+    LONG code;
+    struct Acl *ta;
     struct ViceIoctl blob;
     struct AclEntry *te;
     int idf = 0; //getidf(as, parm_listacl_id);
-
+    char space[AFS_PIOCTL_MAXSIZE];
     HOURGLASS hourglass;
 
-    blob.out_size = MAXSIZE;
+    blob.out_size = AFS_PIOCTL_MAXSIZE;
     blob.in_size = idf;
     blob.in = blob.out = space;
-	
+
     code = pioctl_T(strDir, VIOCGETAL, &blob, 1);
     if (code) {
         ShowMessageBox(IDS_GETRIGHTS_ERROR, MB_ICONERROR, IDS_GETRIGHTS_ERROR, strDir, GetAfsError(errno));
@@ -896,14 +907,14 @@ BOOL GetRights(const CString& strDir, CStringArray& strNormal, CStringArray& str
     return TRUE;
 }
 
-struct AclEntry *FindList(register struct AclEntry *pCurEntry, const char *entryName)
+struct AclEntry *FindList(struct AclEntry *pCurEntry, const char *entryName)
 {
     while (pCurEntry) {
         if (!foldcmp(pCurEntry->name, PCCHAR(entryName)))
             return pCurEntry;
         pCurEntry = pCurEntry->next;
     }
-    
+
     return 0;
 }
 
@@ -911,7 +922,7 @@ void ChangeList(struct Acl *pAcl, BYTE bNormalRights, const CString & entryName,
 {
     ASSERT(pAcl);
     ASSERT(entryName);
-    
+
     struct AclEntry *pEntry;
     CStringUtf8 uEntryName(entryName);
 
@@ -933,10 +944,10 @@ void ChangeList(struct Acl *pAcl, BYTE bNormalRights, const CString & entryName,
     /* Otherwise we make a new item and plug in the new data. */
     pEntry = (struct AclEntry *) malloc(sizeof (struct AclEntry));
     ASSERT(pEntry);
-	
+
     strcpy(pEntry->name, uEntryName);
     pEntry->rights = nEntryRights;
-    
+
     if (bNormalRights) {
         pEntry->next = pAcl->pluslist;
         pAcl->pluslist = pEntry;
@@ -957,20 +968,20 @@ enum rtype {add, destroy, deny};
 
 static LONG Convert(const CString& strRights, int dfs, enum rtype *rtypep)
 {
-    register int i, len;
+    int i, len;
     LONG mode;
 
     *rtypep = add;	/* add rights, by default */
 
-    if (strRights == _T("read")) 
+    if (strRights == _T("read"))
         return PRSFS_READ | PRSFS_LOOKUP;
-    if (strRights == _T("write")) 
+    if (strRights == _T("write"))
         return PRSFS_READ | PRSFS_LOOKUP | PRSFS_INSERT | PRSFS_DELETE | PRSFS_WRITE | PRSFS_LOCK;
-    if (strRights == _T("mail")) 
+    if (strRights == _T("mail"))
         return PRSFS_INSERT | PRSFS_LOCK | PRSFS_LOOKUP;
-    if (strRights == _T("all")) 
+    if (strRights == _T("all"))
         return PRSFS_READ | PRSFS_LOOKUP | PRSFS_INSERT | PRSFS_DELETE | PRSFS_WRITE | PRSFS_LOCK | PRSFS_ADMINISTER;
-    
+
     if (strRights == _T("none")) {
         *rtypep = destroy; /* Remove entire entry */
         return 0;
@@ -992,13 +1003,13 @@ static LONG Convert(const CString& strRights, int dfs, enum rtype *rtypep)
             fprintf(stderr, "illegal rights character '%c'.\n", c);
             exit(1);
         }
-    }   
+    }
     return mode;
 }
 
 BOOL SaveACL(const CString& strCellName, const CString& strDir, const CStringArray& normal, const CStringArray& negative)
 {
-    register LONG code;
+    LONG code;
     struct ViceIoctl blob;
     struct Acl *pAcl;
     LONG rights;
@@ -1033,7 +1044,7 @@ BOOL SaveACL(const CString& strCellName, const CString& strDir, const CStringArr
             ShowMessageBox(IDS_SAVE_ACL_EINVAL_ERROR, MB_ICONERROR, IDS_SAVE_ACL_EINVAL_ERROR, strDir);
         else
             ShowMessageBox(IDS_SAVE_ACL_ERROR, MB_ICONERROR, IDS_SAVE_ACL_ERROR, strDir, GetAfsError(errno, strDir));
-    }       
+    }
 
     ZapAcl(pAcl);
 
@@ -1042,27 +1053,27 @@ BOOL SaveACL(const CString& strCellName, const CString& strDir, const CStringArr
 
 BOOL CopyACL(const CString& strToDir, const CStringArray& normal, const CStringArray& negative, BOOL bClear)
 {
-    register LONG code;
+    LONG code;
     struct ViceIoctl blob;
     struct Acl *pToAcl;
     int idf = 0; // getidf(as, parm_copyacl_id);
-
+    char space[AFS_PIOCTL_MAXSIZE];
     HOURGLASS hourglass;
 
     // Get ACL to copy to
-    blob.out_size = MAXSIZE;
+    blob.out_size = AFS_PIOCTL_MAXSIZE;
     blob.in_size = idf;
     blob.in = blob.out = space;
-	
+
     code = pioctl_T(strToDir, VIOCGETAL, &blob, 1);
     if (code) {
         ShowMessageBox(IDS_ACL_READ_ERROR, MB_ICONERROR, IDS_ACL_READ_ERROR, strToDir, GetAfsError(errno, strToDir));
         return FALSE;
     }
-	
-    if (bClear) 
+
+    if (bClear)
         pToAcl = EmptyAcl(space);
-    else 
+    else
         pToAcl = ParseAcl(space);
 
     if (pToAcl == NULL) {
@@ -1103,7 +1114,7 @@ BOOL CopyACL(const CString& strToDir, const CStringArray& normal, const CStringA
         ZapAcl(pToAcl);
         if (errno == EINVAL)
             ShowMessageBox(IDS_COPY_ACL_EINVAL_ERROR, MB_ICONERROR, IDS_COPY_ACL_EINVAL_ERROR, strToDir);
-        else 
+        else
             ShowMessageBox(IDS_COPY_ACL_ERROR, MB_ICONERROR, IDS_COPY_ACL_ERROR, strToDir, GetAfsError(errno, strToDir));
         return FALSE;
     }
@@ -1137,7 +1148,7 @@ CString ParseMountPoint(const CString strFile, CString strMountPoint)
     strMountPointInfo = strFile + _T("\t") + strVolume + _T("\t") + strCell + _T("\t") + strType;
 
     return strMountPointInfo;
-}       
+}
 
 CString ParseSymlink(const CString strFile, CString strSymlink)
 {
@@ -1146,29 +1157,27 @@ CString ParseSymlink(const CString strFile, CString strSymlink)
     strSymlinkInfo = strFile + _T("\t") + strSymlink;
 
     return strSymlinkInfo;
-}       
+}
 
 BOOL IsPathInAfs(const CString & strPath)
 {
     struct ViceIoctl blob;
+    cm_ioctlQueryOptions_t options;
+    cm_fid_t fid;
     int code;
 
     HOURGLASS hourglass;
 
-    CString debugBuf;
-
-    debugBuf.Format(_T("IsPathInAfs(%s)"), strPath);
-    OutputDebugString(debugBuf);
-
-    blob.in_size = 0;
-    blob.out_size = MAXSIZE;
-    blob.out = space;
-
-    code = pioctl_T(strPath, VIOC_FILE_CELL_NAME, &blob, 1);
-
-    debugBuf.Format(_T("VIOC_FILE_CELL_NAME=%d"), code);
-    OutputDebugString(debugBuf);
+    memset(&options, 0, sizeof(options));
+    options.size = sizeof(options);
+    options.field_flags |= CM_IOCTL_QOPTS_FIELD_LITERAL;
+    options.literal = 1;
+    blob.in_size = options.size;    /* no variable length data */
+    blob.in = &options;
+    blob.out_size = sizeof(cm_fid_t);
+    blob.out = (char *) &fid;
 
+    code = pioctl_T(strPath, VIOCGETFID, &blob, 1);
     if (code) {
 	if ((errno == EINVAL) || (errno == ENOENT))
         return FALSE;
@@ -1176,14 +1185,15 @@ BOOL IsPathInAfs(const CString & strPath)
     return TRUE;
 }
 
-static int 
+static int
 IsFreelanceRoot(const CString& apath)
 {
     struct ViceIoctl blob;
     afs_int32 code;
+    char space[AFS_PIOCTL_MAXSIZE];
 
     blob.in_size = 0;
-    blob.out_size = MAXSIZE;
+    blob.out_size = AFS_PIOCTL_MAXSIZE;
     blob.out = space;
 
     code = pioctl_T(apath, VIOC_FILE_CELL_NAME, &blob, 1);
@@ -1200,7 +1210,7 @@ static const char * NetbiosName(void)
     DWORD dummyLen;
     DWORD enabled = 0;
 
-    code = RegOpenKeyExA(HKEY_LOCAL_MACHINE, AFSREG_CLT_SVC_PARAM_SUBKEY,
+    code = RegOpenKeyEx(HKEY_LOCAL_MACHINE, AFSREG_CLT_SVC_PARAM_SUBKEY,
                          0, (IsWow64()?KEY_WOW64_64KEY:0)|KEY_QUERY_VALUE, &parmKey);
     if (code == ERROR_SUCCESS) {
         dummyLen = sizeof(buffer);
@@ -1282,7 +1292,7 @@ static BOOL IsAdmin (void)
         pszRefDomain = (TCHAR *)malloc(dwSize2);
 
         if (!LookupAccountName(NULL, pszAdminGroup, psidAdmin, &dwSize, pszRefDomain, &dwSize2, &snu)) {
-            /* We can't lookup the group now even though we looked it up earlier.  
+            /* We can't lookup the group now even though we looked it up earlier.
                Could this happen? */
             fAdmin = TRUE;
         } else {
@@ -1384,7 +1394,7 @@ CString Parent(const CString& path)
         }
     }
     }
-    
+
 CString LastComponent(const CString& path)
 {
     int last_slash = path.ReverseFind(_T('\\'));
@@ -1428,7 +1438,7 @@ GetCell(const CString & path)
 
 BOOL ListMount(CStringArray& files)
 {
-    register LONG code;
+    LONG code;
     struct ViceIoctl blob;
     int error;
 
@@ -1436,13 +1446,14 @@ BOOL ListMount(CStringArray& files)
     CStringUtf8 last_component;	        /* Last component of true name */
 
     CStringArray mountPoints;
-    
+
     HOURGLASS hourglass;
 
     error = 0;
 
     for (int i = 0; i < files.GetSize(); i++) {
         int last_slash = files[i].ReverseFind(_T('\\'));
+        char space[AFS_PIOCTL_MAXSIZE];
 
         if (last_slash != -1) {
             last_component.SetString( files[i].Mid(last_slash + 1) );
@@ -1464,16 +1475,18 @@ BOOL ListMount(CStringArray& files)
 
         blob.in_size = last_component.GetLength() + 1;
         blob.in = last_component.GetBuffer();
-        blob.out_size = MAXSIZE;
+        blob.out_size = AFS_PIOCTL_MAXSIZE;
         blob.out = space;
-        memset(space, 0, MAXSIZE);
+        memset(space, 0, AFS_PIOCTL_MAXSIZE);
 
         code = pioctl_T(parent_dir, VIOC_AFS_STAT_MT_PT, &blob, 1);
 
         last_component.ReleaseBuffer();
 
         if (code == 0) {
-            int nPos = strlen(space) - 1;
+            int nPos;
+            space[AFS_PIOCTL_MAXSIZE - 1] = '\0';
+            nPos = strlen(space) - 1;
             if (space[nPos] == '.')
                 space[nPos] = 0;
             mountPoints.Add(ParseMountPoint(StripPath(files[i]), Utf8ToCString(space)));
@@ -1544,7 +1557,7 @@ MakeMount(const CString& strDir,
         ShowMessageBox(IDS_MOUNT_POINT_ERROR, MB_ICONERROR, IDS_MOUNT_POINT_ERROR, GetAfsError(errno, strDir));
         return FALSE;
     }
-    
+
     return TRUE;
 }
 
@@ -1561,7 +1574,7 @@ long fs_ExtractDriveLetter(const char *inPathp, char *outPathp)
     else *outPathp = 0;
 
     return 0;
-}       
+}
 
 /* strip the drive letter from a component */
 long fs_StripDriveLetter(const char *inPathp, char *outPathp, long outSize)
@@ -1577,7 +1590,7 @@ long fs_StripDriveLetter(const char *inPathp, char *outPathp, long outSize)
         strcpy(outPathp, tempBuffer);
     }
     return 0;
-}       
+}
 
 
 BOOL RemoveSymlink(const CString& strName)
@@ -1586,7 +1599,7 @@ BOOL RemoveSymlink(const CString& strName)
     INT code=0;
     struct ViceIoctl blob;
     char lsbuffer[1024];
-    
+
     HOURGLASS hourglass;
 
     CString strParent = Parent(strName);
@@ -1615,21 +1628,15 @@ BOOL RemoveSymlink(const CString& strName)
     ustrLast.ReleaseBuffer();
 
     return (code == 0);
-}       
+}
 
 BOOL IsSymlink(const CString& strName)
 {
     struct ViceIoctl blob;
     int code;
-
+    char space[AFS_PIOCTL_MAXSIZE];
     HOURGLASS hourglass;
 
-	    {
-        CString str;
-        str.Format(_T("IsSymlink(%s)"), strName);
-        OutputDebugString(str);
-    }
-
     CStringUtf8 ustrLast(LastComponent(strName));
     CString strParent = Parent(strName);
 
@@ -1637,47 +1644,31 @@ BOOL IsSymlink(const CString& strName)
 
     blob.in_size = ustrLast.GetLength() + 1;
     blob.in = ustrLast.GetBuffer();
-    blob.out_size = MAXSIZE;
+    blob.out_size = AFS_PIOCTL_MAXSIZE;
     blob.out = space;
-    memset(space, 0, MAXSIZE);
+    memset(space, 0, AFS_PIOCTL_MAXSIZE);
 
     code = pioctl_T(strParent, VIOC_LISTSYMLINK, &blob, 1);
 
     ustrLast.ReleaseBuffer();
 
     return (code==0);
-}       
+}
 
 
 BOOL IsMountPoint(const CString& path)
 {
-    register LONG code = 0;
+    LONG code = 0;
     struct ViceIoctl blob;
     char lsbuffer[1024];
 
     HOURGLASS hourglass;
 
-    {
-        CString str;
-        str.Format(_T("IsMountPoint(%s)"), path);
-        OutputDebugString(str);
-    }
-
     CString parent = Parent(path);
     FixNetbiosPath(parent);
 
     CStringUtf8 mountpoint(LastComponent(path));
 
-	    {
-        CString str;
-#ifdef UNICODE
-        str.Format(_T("last_component=%S"), mountpoint);
-#else
-        str.Format(_T("last_component=%s"), mountpoint);
-#endif
-        OutputDebugString(str);
-    }
-
     blob.in_size = mountpoint.GetLength() + 1;
     blob.in = mountpoint.GetBuffer();
     blob.out = lsbuffer;
@@ -1688,7 +1679,7 @@ BOOL IsMountPoint(const CString& path)
     mountpoint.ReleaseBuffer();
 
     return (code==0);
-}       
+}
 
 
 /*
@@ -1699,7 +1690,7 @@ BOOL IsMountPoint(const CString& path)
  */
 BOOL RemoveMount(CStringArray& files)
 {
-    register LONG code = 0;
+    LONG code = 0;
     struct ViceIoctl blob;
     BOOL error = FALSE;
     CStringArray results;
@@ -1741,7 +1732,7 @@ BOOL RemoveMount(CStringArray& files)
             results.Add(GetMessageString(IDS_ERROR, GetAfsError(errno, StripPath(files[i]))));
         } else
             results.Add(GetMessageString(IDS_DELETED));
-    }   
+    }
 
     LoadString (str, IDS_REMOVE_MP);
     LoadString (str2, IDS_REMOVE_MP_COLUMN);
@@ -1754,11 +1745,11 @@ BOOL RemoveMount(CStringArray& files)
 
 BOOL GetVolumeInfo(CString strFile, CVolInfo& volInfo)
 {
-    register LONG code;
+    LONG code;
     struct ViceIoctl blob;
     struct VolumeStatus *status;
     char *name;
-
+    char space[AFS_PIOCTL_MAXSIZE];
     HOURGLASS hourglass;
 
     volInfo.m_strFilePath = strFile;
@@ -1776,12 +1767,12 @@ BOOL GetVolumeInfo(CString strFile, CVolInfo& volInfo)
 	return TRUE;
      */
 
-    blob.out_size = MAXSIZE;
+    blob.out_size = AFS_PIOCTL_MAXSIZE;
     blob.in_size = 0;
     blob.out = space;
 
     code = pioctl_T(strFile, VIOCGETVOLSTAT, &blob, 1);
-    if (code) {
+    if (code || blob.out_size < sizeof(*status)) {
         volInfo.m_strErrorMsg = GetAfsError(errno, strFile);
         return FALSE;
     }
@@ -1800,17 +1791,17 @@ BOOL GetVolumeInfo(CString strFile, CVolInfo& volInfo)
 
     return TRUE;
 }
-	
+
 BOOL SetVolInfo(CVolInfo& volInfo)
 {
-    register LONG code;
+    LONG code;
     struct ViceIoctl blob;
     struct VolumeStatus *status;
     char *input;
-
+    char space[AFS_PIOCTL_MAXSIZE];
     HOURGLASS hourglass;
 
-    blob.out_size = MAXSIZE;
+    blob.out_size = AFS_PIOCTL_MAXSIZE;
     blob.in_size = sizeof(*status) + 3;	/* for the three terminating nulls */
     blob.out = space;
     blob.in = space;
@@ -1818,7 +1809,7 @@ BOOL SetVolInfo(CVolInfo& volInfo)
     status = (VolumeStatus *)space;
     status->MinQuota = -1;
     status->MaxQuota = volInfo.m_nNewQuota;
-	
+
     input = (char *)status + sizeof(*status);
     *(input++) = '\0';	/* never set name: this call doesn't change vldb */
     *(input++) = '\0';	// No offmsg
@@ -1835,12 +1826,12 @@ BOOL SetVolInfo(CVolInfo& volInfo)
         fprintf(fp, "\t\t\tOther status fields aren't set\n");
         fprintf(fp, "\t\t\t3 nulls follow the VolumeStatus structure.\n");
         fprintf(fp, "\tfollow = 1\n");
-        fclose(fp);		
+        fclose(fp);
     }
 #endif
 
     code = pioctl_T(volInfo.m_strFilePath, VIOCSETVOLSTAT, &blob, 1);
-    if (code) {
+    if (code || blob.out_size < sizeof(*status)) {
         ShowMessageBox(IDS_SET_QUOTA_ERROR, MB_ICONERROR, IDS_SET_QUOTA_ERROR, GetAfsError(errno, volInfo.m_strName));
         return FALSE;
     }
@@ -1857,20 +1848,20 @@ void GetCellName(const CString& cellNamep, struct afsconf_cell *infop)
 
 BOOL CheckServers(const CString& strCellName, WHICH_CELLS nCellsToCheck, BOOL bFast)
 {
-    register LONG code;
+    LONG code;
     struct ViceIoctl blob;
-    register LONG j;
+    LONG j;
     LONG temp = 0;
     struct afsconf_cell info;
     struct chservinfo checkserv;
-
+    char space[AFS_PIOCTL_MAXSIZE];
     HOURGLASS hourglass;
 
     memset(&checkserv, 0, sizeof(struct chservinfo));
     blob.in_size = sizeof(struct chservinfo);
     blob.in = (caddr_t)&checkserv;
 
-    blob.out_size = MAXSIZE;
+    blob.out_size = AFS_PIOCTL_MAXSIZE;
     blob.out = space;
     memset(space, 0, sizeof(afs_int32));	/* so we assure zero when nothing is copied back */
 
@@ -1887,7 +1878,7 @@ BOOL CheckServers(const CString& strCellName, WHICH_CELLS nCellsToCheck, BOOL bF
     }
     if (bFast)
         temp |= 1;	/* set fast flag */
-    
+
     checkserv.magic = 0x12345678;	/* XXX */
     checkserv.tflags = temp;
     checkserv.tinterval = -1;	/* don't change current interval */
@@ -1906,7 +1897,7 @@ BOOL CheckServers(const CString& strCellName, WHICH_CELLS nCellsToCheck, BOOL bF
     }
 
     CStringArray servers;
-    for (j = 0; j < MAXHOSTS; j++) {
+    for (j = 0; j < AFS_MAXHOSTS; j++) {
         memcpy(&temp, space + j * sizeof(LONG), sizeof(LONG));
         if (temp == 0)
             break;
@@ -1920,7 +1911,7 @@ BOOL CheckServers(const CString& strCellName, WHICH_CELLS nCellsToCheck, BOOL bF
     dlg.DoModal();
 
     return TRUE;
-}       
+}
 
 BOOL GetTokenInfo(CStringArray& tokenInfo)
 {
@@ -1970,7 +1961,7 @@ BOOL GetTokenInfo(CStringArray& tokenInfo)
         else {
             rc = ktc_GetToken(&serviceName, &token, sizeof(token), &clientName);
             if (rc) {
-                ShowMessageBox(IDS_GET_TOKENS_UNEXPECTED_ERROR2, MB_ICONERROR, IDS_GET_TOKENS_UNEXPECTED_ERROR2, 
+                ShowMessageBox(IDS_GET_TOKENS_UNEXPECTED_ERROR2, MB_ICONERROR, IDS_GET_TOKENS_UNEXPECTED_ERROR2,
                                 serviceName.name, serviceName.instance,	serviceName.cell, rc);
                 continue;
             }
@@ -1996,10 +1987,10 @@ BOOL GetTokenInfo(CStringArray& tokenInfo)
             else
                 strUserName = userName;
 //		printf("User %s's tokens", userName);
-			
+
 //		printf(" for %s%s%s@%s ", serviceName.name, serviceName.instance[0] ? "." : "",	serviceName.instance, serviceName.cell);
             strCellName = serviceName.cell;
-			
+
             if (tokenExpireTime <= current_time)
                 strExpir = "[>> Expired <<]";
 //		printf("[>> Expired <<]\n");
@@ -2034,13 +2025,6 @@ UINT MakeSymbolicLink(const CString& strName, const CString& strTarget)
     CString strParent = Parent(strName);
     FixNetbiosPath(strParent);
 
-    {
-        CString str;
-        str.Format(_T("MakeSymbolicLink: name = %s target = %s parent = %s\n"),
-                   strName, strTarget, strParent);
-        OutputDebugString(str);
-    }
-
     if ( IsFreelanceRoot(strParent) && !IsAdmin() ) {
 	ShowMessageBox(IDS_NOT_AFS_CLIENT_ADMIN_ERROR, MB_ICONERROR, IDS_NOT_AFS_CLIENT_ADMIN_ERROR);
 	return FALSE;
@@ -2070,8 +2054,8 @@ void ListSymbolicLinkPath(const char *strName,char *strPath,UINT nlenPath)
     char true_name[MAX_PATH+1];		/*``True'' dirname (e.g., symlink target)*/
     char parent_dir[MAX_PATH+1];		/*Parent directory of true name*/
     char *last_component;	/*Last component of true name*/
-    UINT code;    
-
+    UINT code;
+    char space[AFS_PIOCTL_MAXSIZE];
     HOURGLASS hourglass;
 
     strcpy(orig_name, strName);
@@ -2116,22 +2100,22 @@ void ListSymbolicLinkPath(const char *strName,char *strPath,UINT nlenPath)
     }
     blob.in = last_component;
     blob.in_size = strlen(last_component)+1;
-    blob.out_size = MAXSIZE;
+    blob.out_size = AFS_PIOCTL_MAXSIZE;
     blob.out = space;
-    memset(space, 0, MAXSIZE);
+    memset(space, 0, AFS_PIOCTL_MAXSIZE);
     if ((code = pioctl(parent_dir, VIOC_LISTSYMLINK, &blob, 1)))
         strcpy(space,"???");
     ASSERT(strlen(space)<MAX_PATH);
     strncpy(strPath,space,nlenPath);
-}       
+}
 
 BOOL ListSymlink(CStringArray& files)
 {
-    register LONG code;
+    LONG code;
     struct ViceIoctl blob;
     int error;
     CStringArray symlinks;
-    
+    char space[AFS_PIOCTL_MAXSIZE];
     HOURGLASS hourglass;
 
     error = 0;
@@ -2145,17 +2129,21 @@ BOOL ListSymlink(CStringArray& files)
 
         blob.in_size = ustrLast.GetLength() + 1;
         blob.in = ustrLast.GetBuffer();
-        blob.out_size = MAXSIZE;
+        blob.out_size = AFS_PIOCTL_MAXSIZE;
         blob.out = space;
-        memset(space, 0, MAXSIZE);
+        memset(space, 0, AFS_PIOCTL_MAXSIZE);
 
         code = pioctl_T(strParent, VIOC_LISTSYMLINK, &blob, 1);
 
         ustrLast.ReleaseBuffer();
 
         if (code == 0) {
-            CString syml = Utf8ToCString(space);
-            int len = syml.GetLength();
+            CString syml;
+            int len;
+
+            space[AFS_PIOCTL_MAXSIZE - 1] = '\0';
+            syml = Utf8ToCString(space);
+            len = syml.GetLength();
 
             if (len > 0) {
                 if (syml[len - 1] == _T('.'))
