@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -66,7 +66,8 @@ unsigned int VNOSERVERS = 0;
  *	of 0)
  */
 static struct rx_securityClass *
-afs_pickSecurityObject(struct afs_conn *conn, int *secLevel) {
+afs_pickSecurityObject(struct afs_conn *conn, int *secLevel)
+{
     struct rx_securityClass *secObj = NULL;
 
     /* Do we have tokens ? */
@@ -92,15 +93,15 @@ afs_pickSecurityObject(struct afs_conn *conn, int *secLevel) {
  * Try setting up a connection to the server containing the specified fid.
  * Gets the volume, checks if it's up and does the connection by server address.
  *
- * @param afid 
+ * @param afid
  * @param areq Request filled in by the caller.
  * @param locktype Type of lock that will be used.
  *
  * @return The conn struct, or NULL.
  */
 struct afs_conn *
-afs_Conn(register struct VenusFid *afid, register struct vrequest *areq,
-	 afs_int32 locktype)
+afs_Conn(struct VenusFid *afid, struct vrequest *areq,
+	 afs_int32 locktype, struct rx_connection **rxconn)
 {
     u_short fsport = AFS_FSPORT;
     struct volume *tv;
@@ -111,6 +112,8 @@ afs_Conn(register struct VenusFid *afid, register struct vrequest *areq,
     int i;
     struct srvAddr *sa1p;
 
+    *rxconn = NULL;
+
     AFS_STATCNT(afs_Conn);
     /* Get fid's volume. */
     tv = afs_GetVolume(afid, areq, READ_LOCK);
@@ -144,7 +147,7 @@ afs_Conn(register struct VenusFid *afid, register struct vrequest *areq,
      */
     for (notbusy = not_busy; (!lowp && (notbusy <= end_not_busy)); notbusy++) {
 	for (i = 0; i < AFS_MAXHOSTS && tv->serverHost[i]; i++) {
-	    if (((areq->tokenError > 0)||(areq->idleError > 0)) 
+	    if (((areq->tokenError > 0)||(areq->idleError > 0))
 		&& (areq->skipserver[i] == 1))
 		continue;
 	    if (tv->status[i] != notbusy) {
@@ -170,7 +173,7 @@ afs_Conn(register struct VenusFid *afid, register struct vrequest *areq,
     if (lowp) {
 	tu = afs_GetUser(areq->uid, afid->Cell, SHARED_LOCK);
 	tconn = afs_ConnBySA(lowp, fsport, afid->Cell, tu, 0 /*!force */ ,
-			     1 /*create */ , locktype);
+			     1 /*create */ , locktype, rxconn);
 
 	afs_PutUser(tu, SHARED_LOCK);
     }
@@ -195,13 +198,15 @@ afs_Conn(register struct VenusFid *afid, register struct vrequest *areq,
 struct afs_conn *
 afs_ConnBySA(struct srvAddr *sap, unsigned short aport, afs_int32 acell,
 	     struct unixuser *tu, int force_if_down, afs_int32 create,
-	     afs_int32 locktype)
+	     afs_int32 locktype, struct rx_connection **rxconn)
 {
     struct afs_conn *tc = 0;
     struct rx_securityClass *csec;	/*Security class object */
     int isec;			/*Security index */
     int service;
 
+    *rxconn = NULL;
+
     if (!sap || ((sap->sa_flags & SRVR_ISDOWN) && !force_if_down)) {
 	/* sa is known down, and we don't want to force it.  */
 	return NULL;
@@ -220,7 +225,7 @@ afs_ConnBySA(struct srvAddr *sap, unsigned short aport, afs_int32 acell,
 	ReleaseSharedLock(&afs_xconn);
 	return NULL;
     }
-    
+
     if (AFS_IS_DISCONNECTED && !AFS_IN_SYNC) {
         afs_warnuser("afs_ConnBySA: disconnected\n");
         ReleaseSharedLock(&afs_xconn);
@@ -292,9 +297,18 @@ afs_ConnBySA(struct srvAddr *sap, unsigned short aport, afs_int32 acell,
 	if (service == 52) {
 	    rx_SetConnHardDeadTime(tc->id, afs_rx_harddead);
 	}
+	/* set to a RX_CALL_TIMEOUT error to allow MTU retry to trigger */
+	rx_SetServerConnIdleDeadErr(tc->id, RX_CALL_DEAD);
 	rx_SetConnIdleDeadTime(tc->id, afs_rx_idledead);
+	rx_SetMsgsizeRetryErr(tc->id, RX_MSGSIZE);
 
-	rx_SetConnSecondsUntilNatPing(tc->id, 20);
+	/*
+	 * Only do this for the base connection, not per-user.
+	 * Will need to be revisited if/when CB gets security.
+	 */
+	if ((isec == 0) && (service != 52) && !(tu->states & UTokensBad) &&
+	    (tu->vid == UNDEFVID))
+	    rx_SetConnSecondsUntilNatPing(tc->id, 20);
 
 	tc->forceConnectFS = 0;	/* apparently we're appropriately connected now */
 	if (csec)
@@ -302,6 +316,9 @@ afs_ConnBySA(struct srvAddr *sap, unsigned short aport, afs_int32 acell,
 	ConvertWToSLock(&afs_xconn);
     } /* end of if (tc->forceConnectFS)*/
 
+    *rxconn = tc->id;
+    rx_GetConnection(*rxconn);
+
     ReleaseSharedLock(&afs_xconn);
     return tc;
 }
@@ -324,12 +341,15 @@ afs_ConnBySA(struct srvAddr *sap, unsigned short aport, afs_int32 acell,
  */
 struct afs_conn *
 afs_ConnByHost(struct server *aserver, unsigned short aport, afs_int32 acell,
-	       struct vrequest *areq, int aforce, afs_int32 locktype)
+	       struct vrequest *areq, int aforce, afs_int32 locktype,
+	       struct rx_connection **rxconn)
 {
     struct unixuser *tu;
     struct afs_conn *tc = 0;
     struct srvAddr *sa = 0;
 
+    *rxconn = NULL;
+
     AFS_STATCNT(afs_ConnByHost);
 
     if (AFS_IS_DISCONNECTED && !AFS_IN_SYNC) {
@@ -337,7 +357,7 @@ afs_ConnByHost(struct server *aserver, unsigned short aport, afs_int32 acell,
         return NULL;
     }
 
-/* 
+/*
   1.  look for an existing connection
   2.  create a connection at an address believed to be up
       (if aforce is true, create a connection at the first address)
@@ -348,7 +368,7 @@ afs_ConnByHost(struct server *aserver, unsigned short aport, afs_int32 acell,
     for (sa = aserver->addr; sa; sa = sa->next_sa) {
 	tc = afs_ConnBySA(sa, aport, acell, tu, aforce,
 			  0 /*don't create one */ ,
-			  locktype);
+			  locktype, rxconn);
 	if (tc)
 	    break;
     }
@@ -357,7 +377,7 @@ afs_ConnByHost(struct server *aserver, unsigned short aport, afs_int32 acell,
 	for (sa = aserver->addr; sa; sa = sa->next_sa) {
 	    tc = afs_ConnBySA(sa, aport, acell, tu, aforce,
 			      1 /*create one */ ,
-			      locktype);
+			      locktype, rxconn);
 	    if (tc)
 		break;
 	}
@@ -383,19 +403,21 @@ afs_ConnByHost(struct server *aserver, unsigned short aport, afs_int32 acell,
  */
 struct afs_conn *
 afs_ConnByMHosts(struct server *ahosts[], unsigned short aport,
-		 afs_int32 acell, register struct vrequest *areq,
-		 afs_int32 locktype)
+		 afs_int32 acell, struct vrequest *areq,
+		 afs_int32 locktype, struct rx_connection **rxconn)
 {
-    register afs_int32 i;
-    register struct afs_conn *tconn;
-    register struct server *ts;
+    afs_int32 i;
+    struct afs_conn *tconn;
+    struct server *ts;
+
+    *rxconn = NULL;
 
     /* try to find any connection from the set */
     AFS_STATCNT(afs_ConnByMHosts);
     for (i = 0; i < AFS_MAXCELLHOSTS; i++) {
 	if ((ts = ahosts[i]) == NULL)
 	    break;
-	tconn = afs_ConnByHost(ts, aport, acell, areq, 0, locktype);
+	tconn = afs_ConnByHost(ts, aport, acell, areq, 0, locktype, rxconn);
 	if (tconn) {
 	    return tconn;
 	}
@@ -411,16 +433,18 @@ afs_ConnByMHosts(struct server *ahosts[], unsigned short aport,
  * @param locktype
  */
 void
-afs_PutConn(register struct afs_conn *ac, afs_int32 locktype)
+afs_PutConn(struct afs_conn *ac, struct rx_connection *rxconn,
+	    afs_int32 locktype)
 {
     AFS_STATCNT(afs_PutConn);
     ac->refCount--;
+    rx_PutConnection(rxconn);
 }				/*afs_PutConn */
 
 
-/** 
- * For multi homed clients, a RPC may timeout because of a 
- * client network interface going down. We need to reopen new 
+/**
+ * For multi homed clients, a RPC may timeout because of a
+ * client network interface going down. We need to reopen new
  * connections in this case.
  *
  * @param sap Server address.
